/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */


package gdistasi.StarzaNet.server;


import gdistasi.StarzaNet.server.telefono.ConnessioneInternet;
import gdistasi.StarzaNet.server.telefono.Tariffa;
import gdistasi.StarzaNet.server.telefono.TariffaXml;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.StringTokenizer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class Conf {
    String filename;
    static final char carattereCommento='#';
    Document document;
    Node rootNode;
    Node global;
    Tariffa tariffaParagone;
    String dirVar;
    ErrorHandlerMy errorHandler;

    DocumentBuilderFactory factory;
    DocumentBuilder builder;
   
    static public int TYPE_OLD;

    public Conf(String filename,int type){
	this.filename=filename;
    }


    public Conf(String filename,ErrorHandlerMy errorHandler) throws ParserConfigurationException,SAXException,IOException {
	this.filename=filename;
	this.errorHandler=errorHandler;

	factory =  DocumentBuilderFactory.newInstance();
	factory.setIgnoringElementContentWhitespace(true); 
	factory.setValidating(true);
	builder = factory.newDocumentBuilder();
	builder.setErrorHandler(errorHandler);
	document = builder.parse(filename);
	rootNode = document.getDocumentElement();
	parseGlobal();
    }

   
    public void parseGlobal() throws SAXException,ParserConfigurationException,IOException {
	String strTariffa=null;
	Node n;

	global = document.getElementsByTagName("global").item(0);
	
	dirVar=Util.parseSlash(global.getChildNodes().item(0).getFirstChild().getNodeValue());
	
	dirVar=Util.parseSlash(dirVar.trim());

	if (!dirVar.endsWith(File.separator)){
	    dirVar=dirVar+File.separator;
	}

	n=global.getChildNodes().item(1);
	strTariffa=n.getFirstChild().getNodeValue();
	
	tariffaParagone=new TariffaXml(getRootElementTariffa(
			  Environment.getDirTariffe()+strTariffa.trim()),strTariffa,Environment.getLogger());
	
    }

    public String getDirVar(){
	return dirVar;
    }

    public Tariffa getTariffaParagone(){
	return tariffaParagone;
	
    }

    public Tel getTel(){
	NodeList list = document.getElementsByTagName("tel");
	Tel tel=null;
	
	if (list.getLength()>0)
	    tel = new Tel(list.item(0));

	return tel;

    }

    public Firewall getFirewall(){
	NodeList list = document.getElementsByTagName("firewall");

	Firewall fir = new Firewall(list.item(0));
	
	return fir;
    }

    public ArrayList getConnessioni() throws SAXException,ParserConfigurationException,IOException {
	NodeList list = document.getElementsByTagName("connessione");
	ArrayList conn= new ArrayList();
	ConnessioneInternet connessione;

	for (int i=0; i<list.getLength(); i++){
	    connessione= new ConnessioneInternet(list.item(i));
	    conn.add(connessione);
	}

	return conn;
	
    }

    public String[] trovaAr(String chiave) throws IOException,ConfException {
	String str=trova(chiave);
	String[] ret=null;

	if (str!=null) 
	    ret=parseString(str);

	return ret;
    }

    public String trova(String chiave) throws IOException,ConfException {
	String linea;
	String ret=null;
	StringTokenizer token;
	String temp;

	FileInputStream file= new FileInputStream(filename);
	BufferedReader reader=new BufferedReader(new InputStreamReader(file));
	
	while((linea=reader.readLine())!=null) {
	    linea.trim();
	    if (linea.length()>0) { 
		if (linea.charAt(0)==carattereCommento) continue;
		
		token=new StringTokenizer(linea,"=");
		if (token.hasMoreTokens()) { 
		    temp=token.nextToken().trim();
		    if (chiave.equals(temp)) {
			if (!token.hasMoreTokens()) 
			    throw new SecMembroMancante(chiave);
			ret=token.nextToken().trim();
		    }
		    
		}
	    }
	}

	reader.close();
	file.close();
	
	return ret;

    }




    static public String[] parseString(String str) {
	StringTokenizer token=new StringTokenizer(str," ");
	int numToken=token.countTokens();

	String[] ret=new String[numToken];

	for (int i=0; i<numToken; i++)
	    ret[i]=token.nextToken();


	return ret;


    }


     public Element getRootElementTariffa(String filename) throws ParserConfigurationException,SAXException,IOException {
	Node rootNode;
	Document doc;
	doc = builder.parse(filename);
	rootNode = doc.getDocumentElement();
	
	return (Element)rootNode;
	    

    }

    


}


