/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */

package gdistasi.StarzaNet.server;


import gdistasi.StarzaNet.server.gestione.DatiGenerali;
import gdistasi.StarzaNet.server.gestione.FileDatiGenerali;
import gdistasi.StarzaNet.server.telefono.Internet;

import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Connessione implements Runnable {
    UtentiConnessi utentiConnessi;
    DatiGenerali datiGenerali;
    FileDatiGenerali fileDatiGenerali;   
    float spesaAssegnata;
    Internet connessioniInternet;
    AutoTimeout autoTimeout;
    Tel tel;
    Logger log;

    Thread thread;

    public Connessione(Internet internet,Firewall firewall,Tel tel){
	try {
	    this.connessioniInternet=internet;
	    //System.out.println("TEEE"+connessioniInternet);
	    this.tel=tel;
	    //connessioniInternet.stop();
	    this.log=Environment.getLogger();
	    
	    

	    utentiConnessi=new UtentiConnessi(firewall);
	    datiGenerali=new DatiGenerali();
	    autoTimeout=new AutoTimeout(this);


	    fileDatiGenerali=new FileDatiGenerali();
	} catch (IOException e) { 
	    System.err.println(e+ " Err. in avvio!");
	    //System.exit(3);
	}// catch (Exception e){
	// System.err.println(e+" err. in connessione stop()");
	    //System.exit(4);
	//}
	
	thread=new Thread(this);
	thread.start();
    }
    
    public synchronized void attribuisciQuote(){
	float costo;
	long tempo;
	
	costo=connessioniInternet.calcolaCosto();      
	tempo=connessioniInternet.getFine();

	utentiConnessi.ripartisciSpesa(costo-spesaAssegnata,tempo);

	datiGenerali.setCosto(costo);
	datiGenerali.setFine(tempo);
	datiGenerali.setUtentiConnessi(utentiConnessi.getNumUtentiConnessi());

	spesaAssegnata=costo;
     
    }


    public FileDatiGenerali getFileDatiGenerali(){
	return fileDatiGenerali;
    }

    public void run(){
	int count=0;
	while (true){
	    try {
		System.gc();
		Thread.sleep(1000*20);
		
		if (connessioniInternet.isAttiva()){
		    
		    count++;
		    attribuisciQuote();
		    autoTimeout.aggiornaTimeout();
		    controlloTimeout();
		    controllaConnessione();
		    if (count==2){ 
			salvaTutto();		    
			count=0;
		    }
		    
		    if (tel!=null && connessioniInternet.getDurata()>1000*60*2){
			if ((Environment.SQ==false) && (tel.daSquillare())){
			    tel.squilla(connessioniInternet.getInizio());
			    
			}
		    }
		    
		} else {
		    //connessioniInternet.stop(); 
		    utentiConnessiCleanup();
		    autoTimeout.aggiornaTimeout();
		}
		

		
	    } catch (InterruptedException e) {
		System.err.println(e+ " in Conn.java:67");
		System.exit(1);
	    } 
	    
	}
    }
	

    public void controlloTimeout(){
	utentiConnessi.controlloTimeout();
    }

    public synchronized float[] getQuotaFissa(Utente utente){
	float[] valori=new float[2];

	if (connessioniInternet.isAttiva() && utente.isConnesso()) {
	    valori[0]+=datiGenerali.getCostoFissoPerUtente();
	} else {  
	    valori[0]=0;
	    valori[1]=0;
	}

	return valori;

    }

    public int avviaConnessioneInternet(){
	int ris=0;
	long inizio=-1;

	if (tel!=null){
	    tel.preparaSq();
	    if (Environment.SQ==true){
		tel.squillaEdAttendi(connessioniInternet.getInizio());
	    }
	}

	try {
	   connessioniInternet.start();
	   Thread.sleep(2000);
	   if (!connessioniInternet.isAttiva()){ 
	       ris=2;
	       log.severe("Connessione internet non attivata!");
	       return ris;
	   }
	   
	   log.log(Level.INFO,"Connessione ad internet "+connessioniInternet+" avviata");

	   inizio=connessioniInternet.getInizio();

	   datiGenerali = new DatiGenerali(inizio);
	   utentiConnessi.setIdConnessione(inizio);
	   datiGenerali.setNomeConnessione(connessioniInternet.toString());
	   datiGenerali.setInizio(inizio);
	   datiGenerali.setCostoFissoConnessione(connessioniInternet.getCostoFisso());
	   fileDatiGenerali.salva(datiGenerali);
	    
	} catch (IOException e) { 
	    System.err.println( e + "errore in connessione internet");
	    utentiConnessiCleanup();
	    ris=3;
	} catch (InterruptedException e) {
	    System.err.println(e);
	}

	spesaAssegnata=0;
	    
	return ris;
    }

    public void stopConnessione(){
	attribuisciQuote();
	//salvaTutto();
	connessioniInternet.stop();
	utentiConnessi.setIdConnessione(0L);
    }


    public int controllaConnessione(){
	int ris=1;

	try {
	    if (utentiConnessi.getNumUtentiConnessi()<=0){ 
		stopConnessione();
	    }
	} catch (Exception e) {
	    System.err.println(e+ " in connessione.stop()");
	    ris=6;
	} 
	
	return ris;
    }

    public void utentiConnessiCleanup(){
	utentiConnessi.clear();
	autoTimeout.clearNonAuto();
	utentiConnessi.setIdConnessione(0L);
    }

    public void salvaTutto(){
	try {
	    utentiConnessi.mandaMsgPerSalvare();
	    fileDatiGenerali.salva(datiGenerali);
	} catch (IOException e) {
	    System.err.println(e+" err. in salva  tutto");
	}
    }

    public synchronized int connetti(Utente utente){
	int ris=1;

	long timeDiConnUtente=System.currentTimeMillis();


	if (utentiConnessi.getNumUtentiConnessi()==0) {
	    ris=avviaConnessioneInternet();
	    timeDiConnUtente=datiGenerali.getInizio();
	    if (ris!=0) {
 		return ris;
	    }
	}
	
	attribuisciQuote();
	ris=utentiConnessi.aggiungi(utente,timeDiConnUtente);
	controllaConnessione();
	
	if (ris==1){
	    datiGenerali.aggiungiUtentePerCostoFisso();
	    datiGenerali.setCostoFissoConnessione(connessioniInternet.getCostoFisso());
	}
	salvaTutto();

	return ris; 
    }

    public synchronized int disconnetti(Utente utente){
	int ris=1;

	attribuisciQuote();
	salvaTutto();
	ris=utentiConnessi.elimina(utente);
	if (ris!=1) return ris;
	
	datiGenerali.setUtentiConnessi(utentiConnessi.getNumUtentiConnessi());

	controllaConnessione();
	
	        
	return ris; 
    }      

    public void aggiungiMillis(Utente utente,long millis,int ID,boolean autoConnect) {
	autoTimeout.aggiungiMillis(utente,millis,ID,autoConnect);
    }


    public int getNumUtentiConnessi(){
	return utentiConnessi.getNumUtentiConnessi();

    }

    public int timeout(Utente utente){
	return utentiConnessi.aggiornaTimeout(utente);


    }

}

















































































































































