/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */

package gdistasi.StarzaNet.server;


import java.io.BufferedReader;
import java.io.EOFException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.RandomAccessFile;
import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.StringTokenizer;
import java.util.logging.Logger;

public class LettoreConf {
    String filename;
    int parti;
    boolean separatore;
    FileInputStream file;
    BufferedReader buf;
    MessageDigest cripto;
    Logger log;


    final static char CharSeparatore='=';
    final static char passSeparatore=':';

    public LettoreConf(String filename,int parti,boolean separatore) 
    throws NoSuchAlgorithmException,IOException  {
	File file=new File(filename);
	
	log=Environment.getLogger();
	if (!file.exists()){
	    log.severe("Il file degli utenti non esiste. Aggiungere degli utenti.");
	}
	
	this.filename=filename;
	this.parti=parti;
	this.separatore=separatore;
	
    }

    public LettoreConf(String filename,String algoritmo)
	throws NoSuchAlgorithmException,IOException {

	log=Environment.getLogger();

	File file=new File(filename);

	if (!file.exists()){
	    log.severe("Il file degli utenti non esiste! Aggiungere utenti a starzanet");
	}

	this.filename=filename;
	cripto=MessageDigest.getInstance(algoritmo);
    }

    public String[] getParti(String chiave){
	String[] opz = new String[parti];
	
	String str=linea(chiave);
	
	int i=0;

	StringTokenizer tokenizer = new StringTokenizer(str," ");

	while (tokenizer.hasMoreTokens() &&  i<parti ) {
	    opz[i]=tokenizer.nextToken();
	    i++;
	}		
	
	
	return opz;
    }

    public void aggiungiUtente(String user, String pass) throws IOException {
	File file=new File(filename);
	aggiungiUtente(user,pass,file.length());
    }


    public int lengthPass(){
	return cripto.getDigestLength()*2;
    }
    
    public void aggiungiUtente(String user, String pass, long pos) throws IOException {
	RandomAccessFile file= new RandomAccessFile(filename,"rw");

	file.seek(pos);

	byte[] userB=user.getBytes();

	file.write(userB);
	file.write(passSeparatore);
	file.write(crittaPass(pass).getBytes());
	file.write('\n');

	file.close();

	return;
    }


    String getString(byte[] b){

	StringBuffer ret=new StringBuffer(b.length*2+1);
	String r;
	int i=0;
	int q=0;
	int len;
	
	//	System.out.println("B");
	
	while (i<b.length){
	    for (int j=0; j<4; j++){
		q=(int)(b[i]*Math.pow(8,j));
		i++;
		if (i==b.length) break;
    
	    }

	    r=Integer.toHexString(q);
	    len=r.length();

	    for (int w=0; w<8-len; w++){
		ret.append("0");
	    }

	    ret.append(r);

	}

	return ret.toString();
    }


    protected String crittaPass(String pass){
	byte[] bytePass;
	byte[] passCrittata=null;
	String ret=null;

	try {
	    bytePass=pass.getBytes("ISO-8859-1");
	    cripto.update(bytePass);
	    passCrittata=cripto.digest();
	} catch (UnsupportedEncodingException e){
	    System.err.println(e);
	}

	ret=getString(passCrittata);
    
	return ret; 
	
    }

    
    
    public boolean controllaPass(String user, String pass) throws IOException,PassException {
	long index;

	
	StringTokenizer token;
	boolean success=false;
	String passCrittata=crittaPass(pass);
	String t;
	
	index=getIndex(user);
	 
	if (index==-1)
	    throw new PassException();

	RandomAccessFile file=new RandomAccessFile(filename,"r");
	file.seek(index);
	String linea=file.readLine();
	token=new StringTokenizer(linea,":");

	if (token.hasMoreTokens()){
	    if (!(t=token.nextToken()).equals(user)){
		throw new PassException("errore interno nel contr. password");
	    }
	    //System.out.println(t);
	    if (token.hasMoreTokens()){
		t=token.nextToken();
		//		System.out.println(t);
		if (t.equals(passCrittata)){
		    success=true;
		} else throw new PassException();

	    }
	}
	
	file.close();

	return success;
	   
    }

	
	


    

    long getIndex(String user) throws IOException {
	long index=-1;
	RandomAccessFile file=new RandomAccessFile(filename,"r");
	String linea;
	long pos=0;
	StringTokenizer token;

	try {
	    while ((linea=file.readLine())!=null) {
		token=new StringTokenizer(linea,":");
		if (token.hasMoreTokens())
		    if (token.nextToken().equals(user)){
			index=pos;
			break;
		    }
		pos=file.getFilePointer();
	    }
	} catch (EOFException e){
	    System.err.println(e);
	} finally {
	    file.close();
	}

	return index;
    }

    
    

    
    public void cambiaPass(String user, String vecchia, String nuova) throws PassException,IOException  {
	RandomAccessFile file=new RandomAccessFile(filename,"rw");
	String linea;
	boolean stop=false;
	StringTokenizer token;

	controllaPass(user,vecchia);
	
	long index=getIndex(user);

	if (index==-1) { 
	    System.err.println("errore interno in controllo pass");
	    throw new PassException();
	    
	}
	
	aggiungiUtente(user,nuova,index);


    }





    private String linea(String chiave) {
	String str=null;
	String ret=null;
	
	try {
	    file=new FileInputStream(filename);
	    buf=new BufferedReader(new InputStreamReader(file,"ISO-8859-1"));
	
	    while((str=buf.readLine())!=null){
		if (str.indexOf(chiave)==0){ 
		    ret=str;
		    break;
		}
	    }

	    buf.close();
	    file.close();
	
	} catch (IOException e) {System.err.println(e); }

	return ret;

    }

    public static void main (String[] argc) throws Exception {
	LettoreConf leggi=new LettoreConf(argc[0],2,false);
	
	System.out.println(argc[0]);
	
	String[] teah=leggi.getParti("cazzo");

	for (int i=0; i<3; i++){
	    System.out.println(teah[i]+" ");
	    
	}

    }
}









