/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */

package gdistasi.StarzaNet.server;

import gdistasi.StarzaNet.server.comandi.GestoreComandi;
import gdistasi.StarzaNet.server.gestione.FileDatiUtente;

import java.io.IOException;
import java.net.InetAddress;
import java.net.Socket;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;

public class Login implements Runnable {
    MySocket socket;
    InetAddress indirizzo;
    GestoreComandi gestoreComandi;
    ConsolleComandi consolleComandi;
    String user,pass;
    Thread thread;
    static HashMap gruppoFileUtenti=new HashMap(6);
    LettoreConf lettoreConf;
    Logger log;

    static final public String benvenuto="Benvenuto in StarzaNet "+
	Environment.VERSION;
    static final public String msgUser="User: ";
    static final public String msgPass="Pass: ";
    static final public String loginCorretto="Login effettuato correttamente";
    static final public String loginNonCorretto="Login non corretto. Statti buono!";

    public Login(Socket socket,GestoreComandi gestoreComandi,LettoreConf conf) throws ConnessioneCaduta{
	
	this.socket=new MySocket(socket);
	this.gestoreComandi=gestoreComandi;
	this.indirizzo=socket.getInetAddress();

	this.lettoreConf=conf;
	this.log=Environment.getLogger();

	thread=new Thread(this);
	thread.start();
    }

    public void run() {
	try {

	    //System.out.println("ciaciacia");

	    mostraBenvenuto();
	    
	    if (!socket.readLine().equals(Environment.VERSION)){
		socket.writeInt(Environment.Version_Unsuccess);
	    	throw new IOException("Versione del client diversa"); 
	    } else socket.writeInt(Environment.Version_Success);

	    autentica();
	} catch (IOException e) {
	    log.log(Level.INFO,"For user: "+user+ " connessione caduta e socket chiuso",e);
	    socket.close();
	    thread=null;
	} catch (PassException e){
	    log.warning(e.getMessage());
	    socket.writeLine(loginNonCorretto);
	    socket.close();
	} 

    }

    public void mostraBenvenuto(){
	socket.writeLine(benvenuto);
    }

    public void autentica() throws IOException,PassException {    
	    socket.writeLine(msgUser);
	    user=socket.readLine();
	    socket.writeLine(msgPass);
	    pass=socket.readLine();
	    lettoreConf.controllaPass(user,pass);
	    log.log(Level.INFO, "Autenticazione con successo di: "+user);
	    consolleComandi=new ConsolleComandi
		(socket,gestoreComandi,new Utente(user,indirizzo,getFileDatiUtente(user)));
	    consolleComandi.attendiComandi();
		
    }

    
    private synchronized FileDatiUtente getFileDatiUtente(String user) throws IOException {
	FileDatiUtente ret=null;
	if (gruppoFileUtenti.containsKey(user)) {
	    ret=(FileDatiUtente)gruppoFileUtenti.get(user);
	} else {
	    ret=new FileDatiUtente(user);
	    gruppoFileUtenti.put(user,ret);
	    //System.out.println("numero file utente: "+gruppoFileUtenti.size());
	}
	return ret;
    }
    
    private void interrompi(IOException e){
	System.err.println(""+e);
	socket.close();
    }

    public static void main (String[] argc) throws Exception {

    }
}







