/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */
package gdistasi.StarzaNet.server;

import gdistasi.StarzaNet.server.bacheca.Bacheca;
import gdistasi.StarzaNet.server.comandi.GestoreComandi;
import gdistasi.StarzaNet.server.telefono.BestConnessione;
import gdistasi.StarzaNet.server.telefono.ConnessioneInternet;
import gnu.getopt.*;

import java.io.File;
import java.io.IOException;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

public class Main {
    GestoreComandi gestoreComandi;
    Connessione connessione;
    BestConnessione internet;
    ConnessioneInternet conn1,conn2;
    Bacheca bacheca;
    LettoreConf lettoreConf;
    ServerSocket server;
    Socket socket;
    ArrayList connessioni;
    Firewall firewall;
    Tel tel;
    Logger log;

    String baseOption;
    LongOpt[] baseLongOptions;

    public Main() throws Exception{
	this.log=Environment.getLogger();
	baseOption="f:d:";
        baseLongOptions = new LongOpt[1];
	baseLongOptions[0] = new LongOpt("dirconf", LongOpt.REQUIRED_ARGUMENT, null, 2);
    }

    LongOpt[] getBaseLongOptions(){
	return baseLongOptions;
    }
    
    String getBaseOptions(){
	return baseOption;
    }

    public void start(String[] arg) throws Exception {
	
	/*if (System.getSecurityManager() == null) {
            System.setSecurityManager(new SecurityManager());
	    }*/

	parseCommandLine(arg);
	leggiFile(true);
	internet = new BestConnessione(connessioni);
	bacheca=new Bacheca();	    
	connessione=new Connessione(internet,firewall,tel);
	
	try {
	    lettoreConf=new LettoreConf(Environment.getFileUtenti(),"SHA");
	} catch (NoSuchAlgorithmException e){
	    System.err.println(e);
	} catch (IOException e){
	    log.severe(e.getMessage());
	    System.exit(1);
	}

	gestoreComandi=new GestoreComandi(connessione,bacheca,lettoreConf);
	
	try {
	    server=new ServerSocket(Environment.porta,20);
	} catch (BindException e){
	    log.log(Level.SEVERE,e.getMessage());
	    System.err.println(e.getMessage());
	    System.exit(1);
	}
	attendiConnessioni();
    }

    public void parseCommandLine(String[] args){
	
	String toParse="l"+getBaseOptions();

	Getopt g=new Getopt("gesline",args,toParse,getBaseLongOptions());
	int c;
	String arg;
	while ((c = g.getopt()) != -1){

	    if (parseCommonArg(c,g)) continue;
	    
	    switch(c){	    
	    case 'l':
		Environment.SQ=true;
		break;
	    case '?':
		System.exit(1);
		break;
	    default:
		System.out.print("getopt() returned " + c + "\n");
	    }
	}
    }

    boolean parseCommonArg(int c,Getopt g){
	String arg;
	boolean ret=true;

	switch(c){
	case 'f':
	    setFileConf(g.getOptarg());
	    break;
	case 'd':
	    arg = g.getOptarg();
	    Environment.debug=Integer.parseInt(arg);
	    break;
	case 2:
	    Environment.setDirConf(g.getOptarg());
	    break;
	
        default:
	    ret=false;
	    break;
	}

	return ret;
    }

    public void setFileConf(String str){
	File file= new File(str);
	if (!file.exists()){
	    log.severe("Il file "+str+" non esiste");
	    System.exit(1);
	} 
	if (!file.canRead()){
	    log.severe("Non posso leggere "+str);
	    System.exit(1);
	}  
	
	Environment.dirConf=file.getParent()+file.separator;
    }

    public void controllaFile(boolean b){
	File file=new File(Environment.directory);
	
	if (!file.canRead()){
	    log.severe("Non posso leggere "+ Environment.directory);
	    System.exit(1);
	}
	if (b)
	    if (!file.canWrite()){
		log.severe("Non posso scrivere "+ Environment.directory);
		System.exit(1);
	    }
    }
    

    public void leggiFile(boolean b) {
	ErrorHandlerMy errorHandler=new ErrorHandlerMy(Environment.getLogger());
	File file;
	
	try {
	    Conf conf=new Conf(Environment.getFileConf(),errorHandler);
	    Environment.directory=conf.getDirVar();
	    controllaFile(b);
	    Environment.tariffaParagone=conf.getTariffaParagone();
	    connessioni= conf.getConnessioni();
	    firewall = conf.getFirewall();
	    tel = conf.getTel();
	} catch (ParserConfigurationException e){
	    System.err.println(e);
	    System.exit(3);
	} catch (SAXException e){
	    System.err.println(e);
	    System.exit(3);
	} catch (IOException e){
	    System.err.println(e);
	    System.exit(3);
	}
    }

    public void attendiConnessioni(){ 
	while(true)
	    try {
		socket=server.accept();
		Environment.getLogger().log(Level.INFO,
			"Connessione entrante da: "+socket.getInetAddress().toString());
		new Login(socket,gestoreComandi,lettoreConf);
	    } catch (IOException e){ 
		System.err.println(""+e); 
	    }
    }

    public static void main(String[] arg) throws Exception {
	Main m=new Main();
	m.start(arg);
    }
}
