/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */

package gdistasi.StarzaNet.server;



import gdistasi.StarzaNet.server.gestione.Bolletta;
import gdistasi.StarzaNet.server.gestione.Selezionatore;
import gdistasi.util.Consolle;
import gnu.getopt.*;

import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

public class Starzacfg {
    Getopt g;
    ArrayList utenti;
    Selezionatore selezionatore;
    Main main;
    SimpleDateFormat dateFormat;
    String strDateFrom,strDateTo;
    Date dateTo,dateFrom;
    
    public Starzacfg() throws Exception {
	utenti=new ArrayList(5);
	dateFormat=new SimpleDateFormat("dd/MM/yyyy");
	selezionatore=new Selezionatore();
	main=new Main();
    }

    public static void main(String[] arg)  {
	try {
	    Starzacfg m=new Starzacfg();
	    //  m.main.leggiFile(false);
	    m.init(arg);
	} catch (Exception e){
	    System.err.println(e);
	    //	    e.printStackTrace();
	}
    }

    public void init(String[] args) throws IOException {

	String toParse="u:m:"+main.getBaseOptions();
	LongOpt[] baseLong=main.getBaseLongOptions();

	LongOpt[] longopts = new LongOpt[4+baseLong.length];
	longopts[0] = new LongOpt("fromday", LongOpt.REQUIRED_ARGUMENT, null, 10);
	longopts[1] = new LongOpt("today", LongOpt.REQUIRED_ARGUMENT, null, 11);
	longopts[2] = new LongOpt("add", LongOpt.REQUIRED_ARGUMENT, null, 12);

	for (int i=0; i<baseLong.length; i++)
	    longopts[i+3]=baseLong[i];

	g=new Getopt("geslinectl",args,toParse,longopts);
	boolean report=false,cambiaPass=false;
	String utentePerPass="";

	int c;
	String arg;
	while ((c = g.getopt()) != -1){

	    if (main.parseCommonArg(c,g)) continue;

	    switch(c){
	    case 12:
		cambiaPass=true;
		utentePerPass=g.getOptarg();
		break;
	    case 'u':
		addUtente(g.getOptarg());
		break;
	    case 'm':
		selezionatore.addCriterioMese(g.getOptarg());
		break;
	    case 10:
		strDateFrom=g.getOptarg();
		break;
	    case 11:
		strDateTo=g.getOptarg();
		break;

	    case '?':
		System.exit(1);
		break; // getopt() already printed an error
		
		//
	    default:
		System.out.print("getopt() returned " + c + "\n");
	    }

	}

	if (cambiaPass) cambiaPass(utentePerPass);

	boolean bFrom,bTo;
	bFrom=(strDateFrom!=null);
	bTo=(strDateTo!=null);
	

	if ((!bFrom && bTo)||(bFrom&& !bTo)){
	    System.err.println("Indicare sia --fromday che --today");
	    System.exit(1);
	}
	
	try {
	    
	    if (bFrom && bTo){
		dateFrom=dateFormat.parse(strDateFrom);
		dateTo=dateFormat.parse(strDateTo);
		
		selezionatore.addCriterioData(dateFrom,dateTo);
	    }
	    
	    
	    if (utenti.size()>0){
		ErrorHandlerMy errorHandler=new ErrorHandlerMy(Environment.getLogger());
		Conf conf=new Conf(Environment.getFileConf(),errorHandler);
		Environment.directory=conf.getDirVar();
		stampaRiepilogo();
		//	    System.out.println(utenti.size());
	    }
	} catch (java.text.ParseException e){
	    System.err.println(e);
	    System.exit(1);
	} catch (javax.xml.parsers.ParserConfigurationException e){
	    System.err.println(e);
	    System.exit(1);
	} catch (org.xml.sax.SAXException e){
	    System.err.println(e);
	    System.exit(1);
	}
    }

    void addUtente(String utente){
	utenti.add(utente);
    }

    void stampaRiepilogo(){
	Bolletta bolletta=new Bolletta(selezionatore);
	//	System.out.println("bolletta");
	//	System.out.println(utenti);
	bolletta.stampaUtenti(utenti);
    }
    
    
    void cambiaPass(String user) throws IOException {
	String pass;
	LettoreConf lett=null;

	try {
	    System.out.println("File utenti"+Environment.getFileUtenti());
	    lett=new LettoreConf(Environment.getFileUtenti(),"SHA");


	System.out.println("Inserisci password: ");
	pass=Consolle.readString();
	System.out.println("Confermi l'operazione (si o no)?");
	
	
	if (Consolle.readString().toLowerCase().equals("si"))
	    lett.aggiungiUtente(user,pass);
	
	} catch (java.security.NoSuchAlgorithmException e){
	    System.err.println(e);
	    System.exit(1);
	}
	
    }
	

}


