/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */

package gdistasi.StarzaNet.server;

import gdistasi.StarzaNet.server.gestione.DatiUtente;
import gdistasi.StarzaNet.server.gestione.FileDatiUtente;
import gdistasi.StarzaNet.server.telefono.ConnessioneTelefonica;
import gdistasi.StarzaNet.server.telefono.Tariffa;

import java.io.IOException;
import java.net.InetAddress;
import java.util.logging.Logger;

public class Utente {
    protected String user;
    protected InetAddress indirizzo;
    protected boolean connesso;
    protected String email;
    protected DatiUtente datiConnessione;
    protected long tempoPerVerifica;
    protected FileDatiUtente fileConnessioni;
    protected String filePerSalvare;
    protected Tariffa tariffaParagone;
    protected ConnessioneTelefonica conn;
    protected Logger log;
    

    public Utente(String user, InetAddress indirizzo,FileDatiUtente file){
	this.user=user;
	this.indirizzo=indirizzo;
	connesso=false;
	this.log=Environment.getLogger();
	
	tariffaParagone=Environment.getTariffaParagone();
	conn=new ConnessioneTelefonica(tariffaParagone);
	fileConnessioni=file;
	tempoPerVerifica=0;
	
    }
    
    public void setConnesso(long idConnessione,long inizio){
	connesso=true;
	datiConnessione=new DatiUtente(inizio);
	datiConnessione.setIdConnessione(idConnessione);
	tariffaParagone.setInizio(inizio);
	conn.start();
	datiConnessione.setCostoFissoVirtuale(tariffaParagone.getCostoFisso());
	datiConnessione.setConnesso(connesso);
  
	try {
	    fileConnessioni.salva(datiConnessione);
	} catch (IOException e) { System.err.println(e+" in setConnesso Utente.j");
	}
    }

    public void unsetConnesso(){
	connesso=false;
	salvaSuDisco();
	datiConnessione=null;
	conn.stop();
    }

    public void aggiornaTimeout(long time){
	tempoPerVerifica=time;
    }

    public FileDatiUtente getFileDatiUtente(){
	return fileConnessioni;
    }

    public InetAddress getIndirizzo(){
	return indirizzo;
    }

    public long getTimeout(){
	return tempoPerVerifica;
    }

    public String toString(){
	return user;
    }

    public boolean isConnesso(){
	return connesso;

    }

    public void  aggiungiQuota(float costo,long time){
	aggiungiQuota(costo,0,time);

    }

    public void aggiungiQuota(float costo,float risparmio,long fine) {
	if (!isConnesso()) 
	    System.err.println("Errore: Utente.java aggiungiQuota"); 
	else {
	    datiConnessione.aggiungi(costo,risparmio);	
	    datiConnessione.setFine(fine);
	    datiConnessione.setSpesaVirtuale(calcolaSpesaVirtuale(fine));
	}
    }

    public float calcolaSpesaVirtuale(long fine){
	float spesa;
	spesa=conn.calcolaCosto();
	return spesa;
    }

    public  void salvaSuDisco(){
	try {
	    fileConnessioni.salva(datiConnessione);
	} catch (IOException e) { System.err.println(e+" in salvaSuDisco Utente.j");
	}

    }
    
    public DatiUtente getDatiConnessione(){
	return datiConnessione;
    }
 
    public float[] getCosto(){
	float[] costo=new float[2];
	costo[0]=datiConnessione.getCosto();
	costo[1]=datiConnessione.getRisparmio();
	return costo;
    }
   
}










