/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */

package gdistasi.StarzaNet.server;

import java.util.Vector;
import java.util.logging.Level;

public class UtentiConnessi {
    Vector utenti;
    Firewall firewall;
    long idConnessione;
    int precNumDiUtentiConnessi;
    
    static final int timeout=2*60*1000; //3 minuti

    public UtentiConnessi(Firewall firewall){
	utenti=new Vector(4);
	this.firewall=firewall;

    }

    public void setIdConnessione(long id){
	this.idConnessione=id;
    }

    public synchronized void clear(){
	for (int i=0; i<getNumUtentiConnessi(); i++) {
	    elimina(get(i));
	}
    }

    public int getNumUtentiConnessi(){
	return utenti.size();
    }

    public synchronized int aggiungi(Utente utente,long inizio){
	int ris=1;
	if (idConnessione==0) System.err.println("errore in aggiungi utente");
	precNumDiUtentiConnessi=utenti.size();
	if (utenti.indexOf(utente)!=-1) return ris=2;
	else { 
	    utenti.add(utente);
	    utente.aggiornaTimeout(System.currentTimeMillis());
	    utente.setConnesso(idConnessione,inizio);
	    firewall.masq(utente.getIndirizzo());
	    firewall.forward(utente.getIndirizzo());
	    Environment.getLogger().log(Level.INFO,"User "+ utente+" connected from ip "+ utente.getIndirizzo());
	}
	
	return ris; 	
    }
    
    
    public synchronized int elimina(Utente utente){
	int ris=1;
	int ind;
	
	precNumDiUtentiConnessi=utenti.size();
	if ((ind=utenti.indexOf(utente))==-1) return 4;
	else { 
	    utenti.remove(ind);
	    utente.unsetConnesso();
	    firewall.demasq(utente.getIndirizzo());
	    firewall.deforward(utente.getIndirizzo());
	    Environment.getLogger().log(Level.INFO,"User "+ utente+" disconnected from ip "+ utente.getIndirizzo());
	}
	return ris; 
    }     
    
    public synchronized int esegui(String[] comand,Utente utente){
	Runtime runtime=Runtime.getRuntime();
	Process processo;
	int ret=-1;
	int lenComand=comand.length;
	String comando[];
	comando=new String[lenComand+1];

	for (int i=0; i<lenComand; i++){
	    comando[i]=comand[i];
	}
	
	comando[lenComand]=utente.getIndirizzo().getHostAddress();	
	Util.esegui(comando);
		
	return ret;


    }

    public boolean da0a1UtenteConnesso(){
	boolean ris=false;
	if (precNumDiUtentiConnessi==0 && utenti.size()==1)
	    ris=true;
	return ris;

    }

    public boolean connesso(Utente utente){
	return (utenti.indexOf(utente)!=-1);

    }

    public Utente get(int i){
	return (Utente)utenti.get(i);
    }

    public void controlloTimeout(){
	long time=System.currentTimeMillis();
	int ris;

	for (int i=0; i<utenti.size(); i++)
	    if (time-get(i).getTimeout()>timeout){
		ris=elimina(get(i));
		if (ris==4) System.err.println("errore in timeout");
	    }

    }

    

    

    public int aggiornaTimeout(Utente utente){
	int ris=1;

	if (connesso(utente)) {
	    utente.aggiornaTimeout(System.currentTimeMillis());
	} else ris=10;
	
	return ris;

    }

  
    public void ripartisciSpesa(float quota,long tempo){
	float quotaSingola=quota/utenti.size();

	for (int i=0; i<utenti.size(); i++)
	    get(i).aggiungiQuota(quotaSingola,tempo);

	
    }

    public synchronized void mandaMsgPerSalvare(Utente utente){
	if (utenti.indexOf(utente)!=-1)
	    utente.salvaSuDisco();
	else System.err.println("Qualcosa non  va in MandaMsgPerSalvare");

    }

    public synchronized void mandaMsgPerSalvare(){
	for (int i=0; i<utenti.size(); i++)
	    get(i).salvaSuDisco();

    }



    


}
