/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */

package gdistasi.StarzaNet.server.gestione;


import gdistasi.StarzaNet.server.Environment;
import gdistasi.StarzaNet.server.Util;
import gdistasi.StarzaNet.server.telefono.Orario;

import java.io.IOException;
import java.io.OutputStream;
import java.io.PrintStream;
import java.io.RandomAccessFile;
import java.io.Serializable;
import java.text.DecimalFormat;
import java.text.ParsePosition;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;

public abstract class DatiConnessione implements Cloneable,Serializable {
    
    protected long inizio;
    protected long fine;
    protected long inizioNelFile;
    protected float costo;
    protected float risparmio;
    
    static int len=10;

    static String[] campi={"Data","Inizio","Durata","Costo","Risparmio"};

    static DecimalFormat format = new DecimalFormat();
    
    //dd/MM/yy
    //   static protected SimpleDateFormat dataFormat=new SimpleDateFormat("dd/MM/yy");
    //   static protected SimpleDateFormat meseFormat=new SimpleDateFormat("MM");

    

    static public String parseValuta(float valuta){
	format.applyPattern("###0.00");
	return ("E "+format.format(valuta));
    }


    public Class getColumnClass(int col){
	return get(col).getClass();
    }

    public static String getColumnName(int col){
	return campi[col];

    }

    public boolean  equals(Object o){
        DatiConnessione ob=(DatiConnessione)o;
	return 	((DatiConnessione)ob).inizio==inizio;

    }

    public static int getColumnCount(){
	return campi.length;
    }
   
    public String getStringOraInizio(){
	return getStringOraInizio(getInizio());
    }

    public static String getStringOraInizio(long data){
	Orario orario;
	StringBuffer ris=new StringBuffer();

	if (Environment.debug>5){
	    System.out.println("data per pars: "+data);
	}
	
	orario=new Orario(data);
	ris.append(parsePerCron(orario.getOra()));
	ris.append(":");
	ris.append(parsePerCron(orario.getMinuti()));

	if (Environment.debug>5){
	    System.out.println("data per pars: "+data);
	    System.out.print(orario.getOra()+" ");
	    System.out.println(orario.getMinuti());
	}

	return ris.toString();

    }


    public Object get(int col){
	Object ret=null;
	switch(col){
	case 0:
	    ret=dataToString(new Date(inizio));
	    break;
	case 1:
	    ret=getStringOraInizio();
	    break;
	case 2:
	    ret=parseDurata();
	    break;
	case 3:
	    ret=parseValuta(getCostoTotale());
	    break;
	case 4:
	    ret=parseValuta(getRisparmioTotale());
	    break;
	}
	return ret;
    }

  
    public void setRisparmio(float risparmio){
	this.risparmio=risparmio;
    }
    
    
    static void stampaIntestazione(PrintStream out){
	out.println();
	for (int i=0; i<campi.length; i++)
	    out.print(Util.setLength(campi[i],len)+"  ");
	out.println();
    }
    

    void stampaRecord(PrintStream out){
	out.print(Util.setLength(dataToString(new Date(inizio)),len));
	out.print("  ");
	out.print(Util.setLength(getStringOraInizio(),len));
	out.print("  ");
	out.print(Util.setLength(parseDurata(),len));
	out.print("  ");
	out.print(Util.setLength(parseValuta(getCostoTotale()),len)); 
	out.print("  ");
	out.print(Util.setLength(parseValuta(getRisparmioTotale()),len));
	//out.println();
    }
    
    static void stampa(DatiConnessioneS dati,PrintStream out){
	stampa(dati.getArrayList(),out);

    }

    static  void stampa(ArrayList ar,PrintStream out){
	
	stampaIntestazione(out);
	
	for(int i=0; i<ar.size(); i++){
	    ((DatiConnessione)ar.get(i)).stampaRecord(out);
	    System.out.println();
	}	
    }

    

    public float getCosto(){
	return costo;
    }


    public abstract float getCostoTotale();
    public abstract float getRisparmioTotale();

    public String parseDurata(){
	return parseDurata(fine-inizio);
    }

    static public String parsePerCron(int tempo){
	format.applyPattern("00");
	return format.format(tempo);


    }

    static public String parseDurata(long tempo){
	int[] cron=new int[3];
	cron=converti(tempo);
	return (parsePerCron(cron[0])+"h "+
		parsePerCron(cron[1])+"m "+
		parsePerCron(cron[2])+"s");

    }

    public long getInizio(){
	return inizio;
    }

    public void setInizio(long inizio){
	this.inizio=inizio;
	fine=inizio;
	inizioNelFile=-1;
	costo=0;
	risparmio=0;
    }

    public void setSoloInizio(long inizio){
	this.inizio=inizio;
    }

    public long getDurata(){
	return fine-inizio;
    }
   
    public void setFine(long fine){
	this.fine=fine;
    }

    public Object clone(){
	Object o=null;
	try {
	    o=super.clone();
	} catch (CloneNotSupportedException  e) {
	    System.err.println(e + "in datiConnessione");
	}
	return o;
    }


    public long getInizioNelFile(){
	return inizioNelFile;
    }

    public void setInizioNelFile(long pos){
	this.inizioNelFile=pos;
    }

    public float getRisparmio(){
	return risparmio;
    }
    
    public void writeObject(RandomAccessFile s) throws IOException {    
	s.writeLong(inizio);
	s.writeLong(fine);
	s.writeFloat(costo);
	s.writeFloat(risparmio);
    }
    
    public void readObject(RandomAccessFile s) throws IOException {
	inizio=s.readLong();
	fine=s.readLong();
	costo=s.readFloat();
	risparmio=s.readFloat();
    }	

    public DatiConnessione(long inizio, long fine, float costo, float risparmio){
	setInizio(inizio);
	this.costo=costo;
	this.risparmio=risparmio;
	this.fine=fine;
    }


    public DatiConnessione(){};


    public DatiConnessione(long inizio){
	setInizio(inizio);
    }
   
    public void stampa(OutputStream out){
	PrintStream pr=new PrintStream(out);
	
	pr.println("Inizio: "+inizio);
	pr.println("Durata: "+parseDurata());
	pr.println("Costo: "+costo);
	pr.println("Risparmio: "+risparmio);
    }
    
    static public int[] converti(long temp){
	int[] ris=new int[3];
	int millInOra=1000*60*60;
	int millInMinuto=1000*60;
	int millInSecondo=1000;
	long avanzo;

	ris[0]=(int)(temp/millInOra);

	avanzo=temp%millInOra;
	ris[1]=(int)(avanzo/millInMinuto);

	avanzo=avanzo%millInMinuto;
	ris[2]=(int)(avanzo/millInSecondo);
	

	return ris;
    }

    public void setCosto(float costo){
	this.costo=costo;
    }

    public long getFine(){
	return fine;
    }

    public void aggiungi(float costo,float risparmio){
	this.costo+=costo;
	this.risparmio+=risparmio;
    }
    
    static public String dataToString(Date d){
	  SimpleDateFormat dataFormat=new SimpleDateFormat("dd/MM/yy");
   	  
	  return dataFormat.format(d);
	//return (""+d.getTime());
    }

    static public String parseMese(Date data){
	SimpleDateFormat meseFormat=new SimpleDateFormat("MM");

	return meseFormat.format(data);
    }

    static public Date StringToData(String s){
	ParsePosition pos=new ParsePosition(0);
	SimpleDateFormat dataFormat=new SimpleDateFormat("dd/MM/yy");
	return dataFormat.parse(s,pos);
    }

}
















