/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */

package gdistasi.StarzaNet.server.gestione;


import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

public class Selezionatore {
    ArrayList criteri;
    ArrayList criteriData;
    Calendar calendario;

    public static final int TUTTI=-10;
    public static final int MESE_CORRENTE=0;
    public static final int MESE_PREC=-1;
    public static final int MESE_PREC2=-2;
    public static final int MESE_PREC3=-3;
    


    public Selezionatore(){
	criteri=new ArrayList(4);
	criteriData=new ArrayList(2);
	calendario=new GregorianCalendar();
    }

    public void clearCriteri(){
	criteri.clear();
    }

    public void addCriterioMese(int criterio){
	Date curr=new Date(System.currentTimeMillis());
	int anno,mese;

	
	calendario.setTime(curr);
	switch(criterio){

	case(MESE_CORRENTE):
	    break;
	case(MESE_PREC):
	    calendario.add(Calendar.MONTH,-1);
	    break;
	case(MESE_PREC2):
	    calendario.add(Calendar.MONTH,-2);
	    break;
	case(MESE_PREC3):
	    calendario.add(Calendar.MONTH,-3);
	    break;
	}

	anno=calendario.get(Calendar.YEAR);
	mese=calendario.get(Calendar.MONTH);

	criteri.add(new CriterioMese(mese,anno));

    }

    public void addCriterioData(Date from,Date to){
	CriterioData c=new CriterioData(from,to);
	criteriData.add(c);
    }


    public boolean test(DatiConnessione dato){
	long inizio,fine;
	boolean ris=false;
	CriterioMese criterio;
	int anno,mese;

	for (int i=0; i<criteri.size(); i++){
	    criterio=(CriterioMese)criteri.get(i);
	    anno=criterio.getAnno();
	    mese=criterio.getMese();

	 
	    calendario.set(Calendar.YEAR,anno);   
	    calendario.set(Calendar.MILLISECOND,0);
	    calendario.set(Calendar.SECOND,0);
	    calendario.set(Calendar.MINUTE,0);
	    calendario.set(Calendar.HOUR_OF_DAY,0);
	    calendario.set(Calendar.DAY_OF_MONTH,1);
	    calendario.set(Calendar.MONTH,mese);
	 
	    
	    inizio=calendario.getTime().getTime();
	    calendario.add(Calendar.MONTH,1);
	    fine = calendario.getTime().getTime();
	    
	    if ((dato.getInizio()>=inizio) && (dato.getInizio()<fine)){
		ris=true;
		break;
	    }
	}

	Iterator it=criteriData.iterator();
	while (it.hasNext()){
	    CriterioData c=(CriterioData)it.next();
	    if ((dato.getInizio()>=c.getFrom().getTime()) && (dato.getInizio()<c.getTo().getTime())){
		ris=true;
		break;
	    }
	}

	return ris;
    }


    public void addCriterioMese(String str){
	StringTokenizer token=new StringTokenizer(str,",");
	StringTokenizer token2;
	CriterioMese criterio;

	String t1=null,t2;

	int mese,anno;

	//	System.out.println("String in criterio: "+str);

	try {
	    while(token.hasMoreTokens()){
		token2=new StringTokenizer(t1=token.nextToken(),"/");
		mese=Integer.parseInt(t2=token2.nextToken());
		anno=Integer.parseInt(token2.nextToken());
		criterio=new CriterioMese(mese-1,anno);
		//		System.out.println(criterio);
		criteri.add(criterio);
	    }
	} catch (NoSuchElementException e){
	    System.err.println("Errore nel parsing di: "+t1);
	    System.exit(1);
	}
    }

    /*    public Selezionatore(Vector ar){
	this();
	dati=new ArrayList[ar.size()];
	ar.copyInto(dati);
	}*/


}


class CriterioMese {
    int mese;
    int anno;

    public CriterioMese(int mese, int anno) {
	this.mese=mese;
	this.anno=anno;
    }
    
    public int getAnno(){
	return anno;
    }

    public int getMese(){
	return mese;
    }

    public String toString(){
	return ("Criterio: mese: "+mese+" anno: "+anno);
    }

}

class CriterioData {
    Date from;
    Date to;

    public CriterioData(Date from,Date to) {
	this.from=from;
	this.to=to;
    }

    public void setFrom(Date from){
	this.from=from;
    }

    public void setTo(Date to){
	this.to=to;
    }
    
    public Date getFrom(){
	return from;
    }

    public Date getTo(){
	return to;
    }

    public String toString(){
	return ("Criterio: from: "+from+" to: "+to);
    }

}


