/*
 *	Copyright (C) 2002, Giovanni Di Stasi	giovannidistasi@libero.it
 *
 *	This file is part of 'starzanet'
 *
 *	See the file called "LICENSE" for license information.
 */

package gdistasi.StarzaNet.server.telefono;

import gdistasi.StarzaNet.server.Environment;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

public class TariffaXml implements Tariffa {
    float scattoStandard;
    float standard;
    long inizio;
    String name;
    ArrayList fascie;
    boolean flag;
    int con,con2;
    Logger log;
    Vector sconti;
    
    DocumentBuilderFactory factory;
    DocumentBuilder builder;
    Document document;

    public TariffaXml(Element rootNode, String name,Logger log){
	this.log=log;
	this.name=name;
	sconti=new Vector();
	parseRoot(rootNode);
    }
    
   
    
    public TariffaXml(String filename,Logger log) throws SAXException,ParserConfigurationException,IOException  {
	Node rootNode;
	
	sconti=new Vector();
	this.log=log;
	this.name=filename;

	factory = DocumentBuilderFactory.newInstance();
	factory.setIgnoringElementContentWhitespace(true); 
	factory.setValidating(true);
	builder = factory.newDocumentBuilder();
	document = builder.parse(filename);
	rootNode = document.getDocumentElement();
	parseRoot(rootNode);	
    }
    
    
    public void get(Node tariffa){
	Element t=(Element)tariffa;
	NodeList list=t.getElementsByTagName("fascia");
	Node n;
	String value;
	int min;
	float fattore;
	Sconto c;
	    
	for (int i=0; i< list.getLength(); i++){
	    fascie.add(new Fascia(list.item(i)));
	}
	
	n=t.getElementsByTagName("standard").item(0);
	value=n.getFirstChild().getNodeValue();
	standard=Float.parseFloat(value);
	
	list=t.getElementsByTagName("scatto");
	n=list.item(list.getLength()-1);

	scattoStandard=Float.parseFloat(n.getFirstChild().getNodeValue());
    
	list=t.getElementsByTagName("sconto");
	
	for (int i=0; i<list.getLength(); i++){
	    value=list.item(i).getFirstChild().getFirstChild().getNodeValue().trim();
	    min = Integer.parseInt(value);
	    value = list.item(i).getChildNodes().item(1).getFirstChild().getNodeValue().trim();
	    fattore=Float.parseFloat(value);
	    c=new Sconto(min,fattore);
	    sconti.add(c);
	    log.log(Level.INFO,"Aggiunto sconto: "+c);
	}

    }
    
    public void parseRoot(Node rootNode){
	NamedNodeMap nodeMap;
	String strFlag=null;
	Node node;
	
	log.log(Level.INFO,"Caricamento tariffa: "+ name);

	try {
	    if (rootNode.hasAttributes()){
		nodeMap=rootNode.getAttributes();
		for (int i=0; i<nodeMap.getLength(); i++){
		    node=nodeMap.item(i);
		    if (node.getNodeName().equals("flag")){
			strFlag=node.getNodeValue();
			if (Integer.parseInt(strFlag)==1){
			    flag=true;
			    log.log(Level.INFO,"Flag connessione attivato");
			}
			
		    }
		}
	    }
	    
	    fascie=new ArrayList(6);
	    get(rootNode);
	} catch (NumberFormatException e){
	    log.severe("Errore nel parsing: "+e);
	    System.exit(5);
	}
    }

   
    public float getPunteggio(long inizio,long time){
	float punteggio=0;
	long oldInizio=inizio;

	ConnessioneTelefonica conn=new ConnessioneTelefonica(this);
	conn.start();
	punteggio=conn.calcolaCosto(inizio,time);
	punteggio+=getCostoFisso();
	
	inizio=oldInizio;
	return punteggio;
    }
    

    protected Fascia getFascia(long temp){
	Fascia fscelta=null;
	Fascia f;

	Iterator i=fascie.iterator();
	while (i.hasNext()){
	    f=(Fascia)i.next();
	    if (f.isInFascia(temp))
		fscelta=f;
	}

	return fscelta;
    }

    
    public float calcolaCostoDelSecondo(long temp){
	float ret=0;
	Fascia f;
	String scelta=null;
	float retScontata;
	long temp2;

	if (flag) temp2=inizio;
	else temp2=temp;
	
	/*	for (int i=0; i<fascie.size(); i++){
	    if (((Fascia)fascie.get(i)).isInFascia(temp2)){
		fscelta=(Fascia)fascie.get(i);
		ret=fscelta.getCosto();
		//break;


	    }
	    }*/

	f=getFascia(temp2);
	
	if (f==null){ 
	    ret=standard;
	    scelta="standard";
	} else { 
	    scelta=f.toString();
	    ret=f.getCosto();
	}
	
	if (Environment.debug>5){ 
	    con++;
	    
	    if (con==150){ 
		log.log(Level.INFO,"fascia scelta: "+scelta);
		con=0;
	    }
	}

	ret=applicaSconto(ret,temp);
	
	ret=ret/6000; 

	return ret;
    }

    protected float applicaSconto(float costo, long temp){
	Sconto scelto=null;
	Sconto s=null;
	float ret=0;

	for (int i=0; i<sconti.size(); i++){
	    s=(Sconto)sconti.get(i);
	    if ((temp-inizio)>s.getMillis()){
		if (scelto==null) scelto=s;
		else if (scelto.getMillis()>s.getMillis())
		    scelto=s;
	    }
	    
	}

	if (scelto!=null){ 
	    ret=scelto.sconta(costo);
	    if (Environment.debug>5){ 
		con2++;
		if (con2==150){ 
		    log.log(Level.INFO,"Applicato sconto "+scelto);
		    con2=0;
		}
	    }
	} else ret=costo;
	
	return ret;
    }

    public void setInizio(long inizio){
	this.inizio=inizio;
    }
    
    
    public float getCostoFisso(){
	Fascia f;
	float scatto;
	
	if (inizio==0) throw new Error("errore in tariffa... inizio=0");
	f=getFascia(inizio);
	if (f==null || f.getCostoFisso()==-1){
	    scatto=scattoStandard;
	    //	    System.out.println("TEAH"+scatto);
	} else {
	    scatto=f.getCostoFisso();
	}
	    

	return scatto/100;
    }
    
    public String toString(){
	return name;
    }
}

    
class Sconto {
    public int minutes;
    public float fattore;

    public Sconto(int minutes,float fattore){
	this.minutes=minutes;
	this.fattore=fattore;

    }

    public long getMillis(){
	return minutes*60*1000;
    }

    public float sconta(float ret){
	return ret-(ret*fattore);
    }
    
    public String toString(){
	return ("minutes: "+minutes+" fattore: "+fattore);
    }

}
